import os


def add_abspath(rel_path: str = None, file: str = None, dirlist: list = None) -> str:
    rel_path = os.path.abspath(os.path.dirname(rel_path))
    if dirlist is None:
        return os.path.join(rel_path, file)
    return os.path.join(rel_path, *dirlist, file)


import os
from pathlib import Path


def find_meta_file(gml_file: str = None) -> str | None:
    """
    Returns a meta_file file path for a given gml file. Usually a .xsd or .gfs file
    :param gml_file: file path for gml file
    :return: meta_file: file path to meta file or None if not found
    """

    dirname = Path(gml_file).parent

    suffixes = ['.xsd', '.XSD', '.gfs', '.GFS']
    for suffix in suffixes:
        meta_file = Path(dirname, Path(gml_file).stem).with_suffix(suffix)
        if meta_file.exists():
            return str(meta_file)

        continue

    return None




import os


def get_file_split(file: str = None) -> tuple[str | None, str | None, str | None]:
    dirname = os.path.dirname(file)
    filename_no_ext = os.path.splitext(file)[0]
    extension = os.path.splitext(file)[1]
    return dirname, filename_no_ext, extension



import os
from clean_gml.gml import Gml

filepath = 'examples/Vrijvervalleiding.gml'
# filepath = 'example.gml'

# datatype_changes = {'OPENBARE_RUIMTE': 'xsd:float'}
# datatype_changes = {'PROPERTY': 'xsd:float'}

gml = Gml(filepath)
# print(gml.schema.get_parameters_floats())
# gml.datatype_changes(changes=datatype_changes)
gml.replace_commas(ndigits=2)
gml.save()



from . gml_base import GmlBase
from clean_gml.utils import get_file_split


floats = ['xsd:float', 'xsd:decimal', 'double', 'Real']


class GmlSchema(GmlBase):
    """
    Helper class to load the xml schema
    """

    def __init__(self, file):
        super().__init__(file)

    def datatype_changes(self, changes: dict = None):
        if changes is None:
            return None

        if not isinstance(changes, dict):
            raise TypeError('changes must be a dictionary')

        for k, v in changes.items():
            for child in self.root.iter():
                if 'element' in child.tag and child.attrib['name'] == k:
                    child.attrib['type'] = v
                break

    def get_parameters_floats_xsd(self):
        return [child.attrib['name'] for child in self.root.iter() if 'element' in child.tag and child.attrib['type'] in floats]

    def get_parameters_floats(self):
        _, _, extension = get_file_split(self.file)

        params = None

        if extension == '.xsd':
            params = self.get_parameters_floats_xsd()
        elif extension == '.gfs':
            params = self.get_parameters_floats_gfs()

        return params

    def get_parameters_floats_gfs(self):
        names = []
        for name in self.root.findall('.//Name'):
            names.append({'name': name.text})

        # removes properties at top
        names.pop(0)

        types = []
        for name in self.root.findall('.//Type'):
            types.append({'type': name.text})

        c = []
        for name, type_ in zip(names, types):
            temp = {**name, **type_}
            c.append(temp)

        return [item for item in c if item['type'] in floats]

    def save(self):
        super()._save_xml()




from . gml_base import GmlBase
from . gml_schema import GmlSchema
from .. namespace import namespaces
from .. find_meta import find_meta_file


class Gml(GmlBase):
    """
    Class for reading a gml data file as xml data.

    """
    # xsd namespace

    def __init__(self, file):
        super().__init__(file)

        schema_file = find_meta_file(self.file)
        self.schema = GmlSchema(schema_file)

    def datatype_changes(self, changes: dict = None):
        self.schema.datatype_changes(changes=changes)

    def replace_commas(self, ndigits: int = 2):

        g = namespaces.get('g')
        params = [f'{{{g}}}{p}' for p in self.schema.get_parameters_floats()]

        for n in self.root.iter(f'{{{g}}}PROPERTIES'):
            for item in n.iter():
                if item.tag in params:
                    try:
                        item.text = str(round(float(item.text.replace(',', '.')), ndigits))
                    except Exception:
                        print(f'no tag found for item {item}')
                        continue

    def save(self):
        super()._save_xml()
        self.schema.save()




